/**
 * World's voyage
 *
 * @author Marc Oliveras <admin@oligalma.com> 
 * @link http://www.oligalma.com
 * @copyright 2021 Oligalma
 * @license GPL License v3
 */

import java.awt.*;
import java.awt.event.*;
import java.awt.image.BufferedImage;
import javax.swing.*;
import java.io.File;

public class WorldsVoyage extends JApplet implements Runnable, KeyListener {
	int speed = 1;
	int x, y, num = -1;
	boolean ozoneLayer;
	Thread th;
	Image image;
	static BufferedImage buffer;
	private boolean isStandalone = false;

	// Get a parameter value
	public String getParameter(String key, String def) {
		return isStandalone ? System.getProperty(key, def)
				: (getParameter(key) != null ? getParameter(key) : def);
	}

	// Construct the applet
	public WorldsVoyage() {
	}

	// Initialize the applet
	public void init() {
		try {
			this.setSize(new Dimension(640, 400));
			buffer = new BufferedImage(this.getWidth(), this.getHeight(),
					BufferedImage.TYPE_INT_RGB);
			this.addKeyListener(this);
			File directori = new File("images");
			image = Toolkit.getDefaultToolkit().getImage(
					directori.getPath() + System.getProperty("file.separator")
							+ "earth.png");

			x = (this.getWidth() / 2) - 75;
			y = (this.getHeight() / 2) - 75;
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	// Start the applet
	public void start() {
		this.requestFocus();
		if (th == null) {
			th = new Thread(this);
			th.start();
		}
	}

	// Stop the applet
	public void stop() {
	}

	// Destroy the applet
	public void destroy() {
	}

	// Get Applet information
	public String getAppletInfo() {
		return "Applet Information";
	}

	// Get parameter info
	public String[][] getParameterInfo() {
		return null;
	}

	// Main method
	public static void main(String[] args) {
		final WorldsVoyage applet = new WorldsVoyage();
		applet.isStandalone = true;
		
		final JFrame frame = new JFrame();
		// EXIT_ON_CLOSE == 3
		frame.setDefaultCloseOperation(3);
		frame.setTitle("Worlds voyage");
		frame.getContentPane().add(applet, BorderLayout.CENTER);
		applet.init();
		applet.start();
		frame.setSize(640, 400);
		Dimension d = Toolkit.getDefaultToolkit().getScreenSize();
		frame.setLocation((d.width - frame.getSize().width) / 2,
				(d.height - frame.getSize().height) / 2);
		frame.setVisible(true);

		KeyListener kl = new KeyListener() {
			public void keyPressed(KeyEvent e) {
				applet.keyPressed(e);
			}

			public void keyReleased(KeyEvent e) {

			}

			public void keyTyped(KeyEvent e) {
			}
		};

		frame.addKeyListener(kl);

		frame.addComponentListener(new ComponentListener() {
		    public void componentResized(ComponentEvent e) {
		        applet.setSize(new Dimension(frame.getWidth(), frame.getHeight() - (frame.getHeight() - applet.getHeight())));
				buffer = new BufferedImage(frame.getWidth(), frame.getHeight(),
						BufferedImage.TYPE_INT_RGB);
			}

			@Override
			public void componentHidden(ComponentEvent arg0) {
				// TODO Auto-generated method stub
				
			}

			@Override
			public void componentMoved(ComponentEvent arg0) {
				// TODO Auto-generated method stub
				
			}

			@Override
			public void componentShown(ComponentEvent arg0) {
				// TODO Auto-generated method stub
				
			}
		});
	}

	public void paint(Graphics g) {
		Graphics2D g2 = (Graphics2D) buffer.createGraphics();

		g2.setColor(Color.black);
		g2.fillRect(0, 0, this.getWidth(), this.getHeight());

		g2.setColor(Color.white);
		for (int i = 0; i < 10; i++)
			g2.fillOval((int) (Math.random() * this.getWidth()),
					(int) (Math.random() * this.getHeight()), 4, 4);
		
		if (image != null)
			g2.drawImage(image, x, y, this);

		if (ozoneLayer) {
			int margin = 5;
			g2.setColor(Color.red);
			g2.drawOval(x - margin - 6, y - margin - 6, image.getWidth(this) + 2 * (margin + 6), image.getHeight(this) + 2 * (margin + 6));
			g2.setColor(Color.yellow);
			g2.drawOval(x - margin - 4, y - margin - 4, image.getWidth(this) + 2 * (margin + 4), image.getHeight(this) + 2 * (margin + 4));
			g2.setColor(Color.red);
			g2.drawOval(x - margin - 2, y - margin - 2, image.getWidth(this) + 2 * (margin + 2), image.getHeight(this) + 2 * (margin + 2));
		}

		g.drawImage(buffer, 0, 0, this);
	}

	public void run() {
		if (Thread.currentThread() == th) {
			for (;;) {
				if (x + image.getWidth(this) > this.getWidth()) {
					if (num == 3)
						num = 1;
					else if (num == 6)
						num = 4;
					else if (num == 9)
						num = 7;
				} else if (x < 0) {
					if (num == 1)
						num = 3;
					else if (num == 4)
						num = 6;
					else if (num == 7)
						num = 9;
				} else if (y + image.getHeight(this) > this.getHeight()) {
					if (num == 1)
						num = 7;
					else if (num == 2)
						num = 8;
					else if (num == 3)
						num = 9;
				} else if (y < 0) {
					if (num == 7)
						num = 1;
					else if (num == 8)
						num = 2;
					else if (num == 9)
						num = 3;
				}

				if (num == 0)
					;
				else if (num == 1) {
					x -= speed;
					y += speed;
				} else if (num == 2)
					y += speed;
				else if (num == 3) {
					x += speed;
					y += speed;
				} else if (num == 4)
					x -= speed;
				else if (num == 6)
					x += speed;
				else if (num == 7) {
					x -= speed;
					y -= speed;
				} else if (num == 8)
					y -= speed;
				else if (num == 9) {
					x += speed;
					y -= speed;
				}
				
				try {
					th.sleep(5);
				} catch (InterruptedException e) {
				}
				repaint();
			}
		}
	}

	@Override
	public void keyPressed(KeyEvent e) {
		e.setKeyChar(Character.toUpperCase(e.getKeyChar()));
		
		int codi = e.getKeyChar();
		if (codi == KeyEvent.VK_0)
			num = 0;
		else if (codi == KeyEvent.VK_1)
			num = 1;
		else if (codi == KeyEvent.VK_2)
			num = 2;
		else if (codi == KeyEvent.VK_3)
			num = 3;
		else if (codi == KeyEvent.VK_4)
			num = 4;
		else if (codi == KeyEvent.VK_5)
			ozoneLayer = !ozoneLayer;
		else if (codi == KeyEvent.VK_6)
			num = 6;
		else if (codi == KeyEvent.VK_7)
			num = 7;
		else if (codi == KeyEvent.VK_8)
			num = 8;
		else if (codi == KeyEvent.VK_9)
			num = 9;
		else if (codi == KeyEvent.VK_A)
		{
			if(speed < 10)
			{
				speed++;
			}
		}
		else if (codi == KeyEvent.VK_Z)
		{
			if(speed > 1)
			{
				speed--;
			}
		}
	}

	@Override
	public void keyReleased(KeyEvent arg0) {
	}

	@Override
	public void keyTyped(KeyEvent arg0) {
	}
}